# -*- coding: utf-8 -*-
""" capXNode.py
    Copyright 2011 Bernd Jungmann
>>> ScoreExportXNode
    dieses Skript enthält Hilfsfunktionen für capella-Braille-Export.
    Nicht direkt aufrufen.
<<<
"""

from caplib.rational import Rational



#--- Hilfsfunktionen zur Suche von Kind-Elementen:


class ScoreExportXNode:
    def __init__(self, outputFile):
        messageBox("ScoreExportXNode", "init")
        self.f = file(outputFile, 'wt')
        self.root = rootNode()
        n = self.root.getChildNodeCount()
        for i in range(n):
            node = self.root.getChildNodeI(i)
            if node.getName() == 'score':
                self.handleScore(node)

    def encode(self, s):
#        return '&amp;'.join([x.replace('"', '&quot;').replace('<','&lt;')
#                             for x in s.split('&')]).encode(self.doc.encoding)
        return s

    def decode(self, s):
        return s

    def handleScore(self, score):
        self.startScore(score)
        n = score.getChildNodeCount()
        for i in range(n):
            node = score.getChildNodeI(i)            
            if node.getName() == 'systems':
                nSys = node.getChildNodeCount()
                for self.iSys in range(nSys):
                    nodeSys = node.getChildNodeI(self.iSys)
                    if nodeSys.getName() == 'system':
                        self.handleSystem(nodeSys)

        self.finishScore(score)

    def startScore(self, score): pass
    def finishScore(self, score): pass

    def handleSystem(self, sys):
        self.startSystem(sys)

        n = sys.getChildNodeCount()
        for i in range(n):
            node = sys.getChildNodeI(i)
            if node.getName() == 'staves':
                nStv = node.getChildNodeCount()
                for self.iStv in range(nStv):
                    nodeStv = node.getChildNodeI(self.iStv)
                    if nodeStv.getName() == 'staff':
                        self.handleStaff(nodeStv)

        self.finishSystem(sys)

    def startSystem(self, sys): pass
    def finishSystem(self, sys): pass

    def handleStaff(self, staff):
        self.startStaff(staff)

        n = staff.getChildNodeCount()
        for i in range(n):
            node = staff.getChildNodeI(i)
            if node.getName() == 'voices':
                nVc = node.getChildNodeCount()
                for self.iVc in range(nVc):
                    nodeVc = node.getChildNodeI(self.iVc)
                    if nodeVc.getName() == 'voice':
                        self.handleVoice(nodeVc)

        self.finishStaff(staff)

    def startStaff(self, staff): pass
    def finishStaff(self, staff): pass

    def handleVoice(self, voice):
        self.startVoice(voice)
        n = voice.getChildNodeCount()
        for i in range(n):
            node = voice.getChildNodeI(i)
            if node.getName() == 'noteObjects':
                nObj = node.getChildNodeCount()
                for self.iObj in range(nObj):
                    nodeObj = node.getChildNodeI(self.iObj)
                    self.handleNoteObj(nodeObj)

        self.finishVoice(voice)

    def startVoice(self, voice): pass
    def finishVoice(self, voice): pass

    def handleNoteObj(self, obj):
        self.startNoteObj(obj)
        if obj.getName() == 'chord':
            self.handleChord(obj)
        elif obj.getName() == 'rest':
            self.handleRest(obj)
        elif obj.getName() == 'clefSign':
            self.handleClefSign(obj)
        elif obj.getName() == 'keySign':
            self.handleKeySign(obj)
        elif obj.getName() == 'timeSign':
            self.handleTimeSign(obj)
        self.finishNoteObj(obj)

    def startNoteObj(self, obj): pass
    def finishNoteObj(self, obj): pass

    def handleChord(self, obj):
        self.startChord(obj)
        n = obj.getChildNodeCount()
        for i in range(n):
            node = obj.getChildNodeI(i)
            if node.getName() == 'heads':
                nHead = node.getChildNodeCount()
                for iHead in range(nHead):
                    nodeHead = node.getChildNodeI(iHead)
                    if nodeHead.getName() == 'head':
                        self.handleHead(nodeHead)
        self.finishChord(obj)

    def startChord(self, chord): pass
    def finishChord(self, chord): pass

    def handleRest(self, rest): pass
    def handleClefSign(self, clefSign): pass
    def handleKeySign(self, keySign): pass
    def handleTimeSign(self, timeSign): pass
    
    def handleHead(self, head): pass

    def midiPitch(self, head):
        attrs = head.getAttributes()
        p = attrs['pitch']
        i = 'CDEFGAB'.find(p[0])
        n = (0,2,4,5,7,9,11)[i] + 12 * int(p[1])
        if attrs.has_key('alter'):
            step = attrs['step']
            if step != '':
                n = n + int(step)
        return n

    def duration(self, note):
        """ Dauer einer Note oder Pause als rationale Zahl in ganzen Noten
            <note> muss ein 'chord'- oder 'rest'-Knoten sein
        """
        duration = note.getChildNode('duration',0)
        attrs = node.getAttributes()
        if attrs.has_key('noDuration') and attrs['noDuration'] == 'true':
            return 0
        n = Rational(str(attrs['base']))

        if attrs.has_key('dots'):
            dots = attrs['dots']
        if dots != '':
            if int(dots) == 1: n = (3 * n) / 2
            else: n = (7 * n) / 4

        tuplet = note.getChildNode('tuplet',0)
        if not tuplet.isEmpty():
            attrsT = tuplet.getAttributes()
            denominator = Rational(int(attrsT['count']))
            numerator = Rational(1)
            if attrsT.has_key('tripartite') and attrsT['tripartite'] == 'true':
               numerator = Rational(3,2)
            while numerator < denominator: numerator *= 2
            if attrsT.has_key('prolong') and attrsT['prolong'] != 'true':
               numerator /= 2
            n = (n * numerator) / denominator

        return n


